# Generated by Django 5.0.8 on 2025-01-10 12:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_chats", "0007_messagestatus_increment_unread_count"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="chatroomparticipant",
            options={"ordering": ["-role", "profile__name"]},
        ),
    ]
